const charactersSet = require('../src/datasets/got/characters.json')
const genderSet = require('../src/datasets/got/genders.json')
const groupSet = require('../src/datasets/got/groups.json')
const fs = require('fs')
const generateUuid = require('uuid').v4

const findCharacterGender = name => (genderSet.male.includes(name) ? 'male' : 'female')

const findCharacterGroup = name => groupSet.groups.find(group => group.characters.includes(name))
const findCharacter = name =>
  charactersSet.characters.find(character => character.characterName === name)
const findCharacterByGroup = houseName =>
  charactersSet.characters.find(character => character.houseName === houseName)

const getGroupName = group => (group !== undefined ? group.name : 'no group')
const getDirection = relationship =>
  ['killed', 'parentOf', 'abducted'].includes(relationship) ? true : false

const onlyUnique = (value, index, self) => self.indexOf(value) === index

;(async () => {
  let dataset = {
    nodes: [],
    links: []
  }
  let characterNames = []
  const relationships = ['killed', 'parentOf', 'siblings', 'marriedEngaged', 'allies', 'abducted']

  // build an array of characters involved
  relationships.forEach(function(relationship) {
    charactersSet.characters.forEach(function(character) {
      if (character.hasOwnProperty(relationship)) {
        characterNames.push(character.characterName)
        character[relationship].forEach(function(v) {
          characterNames.push(v)
        })
      }
    })
  })

  characterNames = characterNames.filter(onlyUnique).sort()
  // console.log(characterNames)
  // console.log(
  //   characterNames
  //     .map(c => {
  //       if (findCharacterGroup(c)) return c
  //     })
  //     .filter(c => {
  //       return c !== undefined
  //     })
  // )
  characterNames = characterNames
    .map(c => {
      if (findCharacterGroup(c)) return c
    })
    .filter(c => {
      // console.log(c)
      return c !== undefined // && c !== "Night's Watch" && c !== "Wildlings" && c !== "Frey"
      // return ![undefined, "Night's Watch"].includes(c) // !== undefined && c !== "Night's Watch" && c !== "Wildlings" && c !== "Frey"
    })
  // console.log(characterNames.map( c => findCharacterGroup(c).name))
  characterNames = characterNames.filter(c => {
    return [
      'Stark',
      'Tyrell',
      'Lannister',
      'Martell',
      'Baratheon',
      'Include',
      'Targaryen'
    ].includes(findCharacterGroup(c).name)
  })

  characterNames = characterNames.filter(c => {
    if (findCharacterGroup(c).name === 'Include') {
      if (
        [
          'Brienne of Tarth',
          'Bronn',
          'Gendry',
          'Grand Maester Pycelle',
          'Gregor Clegane',
          'Lord Varys',
          'Petyr Baelish',
          'Sandor Clegane'
        ].includes(c)
      ) {
        return c
      }
    } else {
      return c
      // console.log(findCharacterGroup(c).name)
    }
  })

  characterNames = characterNames.filter(
    c =>
      ![
        'Benjen Stark',
        'Drogon',
        'Ghost',
        'Grey Wind',
        'Kevan Lannister',
        'Lady',
        'Lancel Lannister',
        "Selyse Baratheon",
        "Rhaegal",
        "Shaggydog",
        "Shireen Baratheon",
        "Summer",
        "Tyene Sand",
        "Viserion",
        "Viserys Targaryen",
        "Nymeria Sand",
        "Obara Sand"
      ].includes(c)
  )
  // console.log(test)



  // console.log(characterNames)
  // characterNames = characterNames.filter(c => getGroupName(findCharacterGroup(findCharacter(c))) === 'Stark')
  // console.log(findCharacterByGroup('Stark'))
  dataset.nodes = characterNames.map((character, id) => {
    const data = findCharacter(character)
    return {
      id,
      uuid: generateUuid(),
      name: character,
      label: character,
      gender: findCharacterGender(character),
      group: getGroupName(findCharacterGroup(character)),
      dataClass: 'person',
      type: 'regular',
      royal: data !== undefined ? data.royal : false,
      kingsguard: data !== undefined ? data.kingsguard : false
    }
  })

  // build list of links based on position in array of characters
  relationships.forEach(function(relationship) {
    charactersSet.characters.forEach(function(character) {
      if (character.hasOwnProperty(relationship)) {
        character[relationship].forEach(function(v) {
          const source = characterNames.indexOf(v)
          const target = characterNames.indexOf(character.characterName)
          if (source === target) return
          if (source === -1 || target === -1) return
          if (!getDirection(relationship)) {
            let res = dataset.links.find(function(link) {
              // returns true if it already exists, just opposite
              return (
                link.source === source &&
                link.target === target &&
                link.description === relationship
              )
            })
            if (res !== undefined) return
          }
          dataset.links.push({
            uuid: generateUuid(),
            source : target, // otherwise, direction of directional link is opposite
            target : source,
            description: relationship,
            direction: getDirection(relationship),
            id: Math.floor(Math.random() * 1000000000)
          })
        })
      }
    })
  })

  // dataset.links = dataset.links.filter(
  //   lLink =>
  //     !dataset.links.find(rLink => rLink.source === lLink.target || rLink.target === lLink.source)
  // )
  fs.writeFile(
    'src/datasets/got.myv4.js',
    `/* eslint-disable */\nexport default ${JSON.stringify(dataset, null, 2)}`,
    'utf8',
    () => {}
  )
})()
